/**
 * This file implements misc useful functions.
 */
goog.provide('common.helpers');

common.helpers.isArray = function(x){
  if(typeof x === 'object'){
    if(x instanceof Array){return true;}
    else{return false;}
  }else{return false;}
};

common.helpers.isEmptyString = function(str){
  return (str==='');
};

common.helpers.getElementByTagAndClassName = function(tagName, className) {
  var elements = document.getElementsByTagName(tagName);
  var myclass = new RegExp('\\b'+className+'\\b');
  var retnode = [];
  for (var i = 0; i < elements.length; ++i) {
    var classes = elements[i].className;
    if (myclass.test(classes))
      retnode.push(elements[i]);
  }
  return retnode;
};

/**
 * Returns the initials, given the full name of a person
 */
common.helpers.getInitials = function(fullName){
  var firstChar='';
  if(fullName.length>0){
    firstChar=fullName.charAt(0);
  }
  var spaceIdx=fullName.indexOf(' ');
  var secondChar='';
  if(spaceIdx!==-1 && (spaceIdx+1)<fullName.length){
    secondChar=fullName.charAt(spaceIdx+1);
  }
  return firstChar.concat(secondChar);
};

common.helpers.getFirstName = function(fullName){
  var spaceIdx=fullName.indexOf(' ');
  if(spaceIdx===-1){spaceIdx=fullName.length;};
  return fullName.slice(0,spaceIdx);
};

common.helpers.virtualErrorFn = function(){
  throw Error('Calling a virtual function not allowed\n');
};

/**
 * This function is supposed to take a string and return a summary
 * of it so that it can fit in a smaller space.
 * TODO: Check if goog library has a function for this.
 */
common.helpers.shortenText = function(text,maxLen){
  goog.asserts.assert(typeof maxLen === 'number');
  maxLen=Math.min(text.length,maxLen);
  return text.slice(0,maxLen); // this is a very dumb summary right now
};

/**
 * Return profile url given a facebook id
 */
common.helpers.getProfileUrl = function(id){
  return ('http://www.facebook.com/profile.php?id='+id);
};

/**
 * allFields and countFields are arrays of objects, and
 * they both have the field idxField. The idxField in countFields
 * should be a subset of the idxFields of allFields
 * Returns a hash map hMap, where hMap[idxField] counts how many
 * times idxField occurs in countFields array.
 */
common.helpers.buildCountHash = function(idxField,allFields,countFields){
  var hMap={};
  for(var i=0;i<allFields.length;i++){
    hMap[allFields[i][idxField]] = 0;
  }
  for(var i=0;i<countFields.length;i++){
    hMap[countFields[i][idxField]]++;
  }
  return hMap;
};

/**
 * Sets the text of DOM element in browser independent way
 */

common.helpers.setText = function (element,text) {
    if (element.innerText != undefined) {
        element.innerText = text
      } else {
        element.textContent = text;
      }
};

/**
 * Remove all children of an element
 */

common.helpers.clearChildren = function (element) {
  while(element.hasChildNodes())
    element.removeChild(element.firstChild);
};

/**
 * Get Random string
 */

common.helpers.randomString = function () {
  var chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz";
  var string_length = 8;
  var randomstring = '';
  for (var i=0; i<string_length; i++) {
    var rnum = Math.floor(Math.random() * chars.length);
    randomstring += chars.substring(rnum,rnum+1);
  }
  return randomstring;
};
